@@ -155,7 +155,7 @@ gtk_tree_model_base_init (gpointer g_class)
        * This signal is emitted when a row in the model has changed.
        */
       tree_model_signals[ROW_CHANGED] =
-        g_signal_new (I_("row_changed"),
+        g_signal_new (I_("row-changed"),
                       GTK_TYPE_TREE_MODEL,
                       G_SIGNAL_RUN_LAST, 
                       G_STRUCT_OFFSET (GtkTreeModelIface, row_changed),
@@ -193,7 +193,7 @@ gtk_tree_model_base_init (gpointer g_class)
       closure = g_closure_new_simple (sizeof (GClosure), NULL);
       g_closure_set_marshal (closure, row_inserted_marshal);
       tree_model_signals[ROW_INSERTED] =
-        g_signal_newv (I_("row_inserted"),
+        g_signal_newv (I_("row-inserted"),
                        GTK_TYPE_TREE_MODEL,
                        G_SIGNAL_RUN_FIRST,
                        closure,
@@ -212,7 +212,7 @@ gtk_tree_model_base_init (gpointer g_class)
        * its last child row.
        */
       tree_model_signals[ROW_HAS_CHILD_TOGGLED] =
-        g_signal_new (I_("row_has_child_toggled"),
+        g_signal_new (I_("row-has-child-toggled"),
                       GTK_TYPE_TREE_MODEL,
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (GtkTreeModelIface, row_has_child_toggled),
@@ -242,7 +242,7 @@ gtk_tree_model_base_init (gpointer g_class)
       closure = g_closure_new_simple (sizeof (GClosure), NULL);
       g_closure_set_marshal (closure, row_deleted_marshal);
       tree_model_signals[ROW_DELETED] =
-        g_signal_newv (I_("row_deleted"),
+        g_signal_newv (I_("row-deleted"),
                        GTK_TYPE_TREE_MODEL,
                        G_SIGNAL_RUN_FIRST,
                        closure,
@@ -257,6 +257,9 @@ gtk_tree_model_base_init (gpointer g_class)
        * @path: a #GtkTreePath identifying the tree node whose children
        *        have been reordered
        * @iter: a valid #GtkTreeIter pointing to the node whose 
+       * @new_order: an array of integers mapping the current position of
+       *             each child to its old position before the re-ordering,
+       *             i.e. @new_order<literal>[newpos] = oldpos</literal>.
        *
        * This signal is emitted when the children of a node in the #GtkTreeModel
        * have been reordered. 
@@ -268,7 +271,7 @@ gtk_tree_model_base_init (gpointer g_class)
       closure = g_closure_new_simple (sizeof (GClosure), NULL);
       g_closure_set_marshal (closure, rows_reordered_marshal);
       tree_model_signals[ROWS_REORDERED] =
-        g_signal_newv (I_("rows_reordered"),
+        g_signal_newv (I_("rows-reordered"),
                        GTK_TYPE_TREE_MODEL,
                        G_SIGNAL_RUN_FIRST,
                        closure,
@@ -850,10 +853,11 @@ gtk_tree_path_down (GtkTreePath *path)
  * gtk_tree_iter_copy:
  * @iter: A #GtkTreeIter.
  *
- * Creates a dynamically allocated tree iterator as a copy of @iter.  This
- * function is not intended for use in applications, because you can just copy
- * the structs by value (<literal>GtkTreeIter new_iter = iter;</literal>).  You
- * must free this iter with gtk_tree_iter_free ().
+ * Creates a dynamically allocated tree iterator as a copy of @iter.  
+ * This function is not intended for use in applications, because you 
+ * can just copy the structs by value 
+ * (<literal>GtkTreeIter new_iter = iter;</literal>).
+ * You must free this iter with gtk_tree_iter_free().
  *
  * Return value: a newly-allocated copy of @iter.
  **/
@@ -874,8 +878,8 @@ gtk_tree_iter_copy (GtkTreeIter *iter)
  * gtk_tree_iter_free:
  * @iter: A dynamically allocated tree iterator.
  *
- * Frees an iterator that has been allocated on the heap.  This function is
- * mainly used for language bindings.
+ * Frees an iterator that has been allocated by gtk_tree_iter_copy().
+ * This function is mainly used for language bindings.
  **/
 void
 gtk_tree_iter_free (GtkTreeIter *iter)
@@ -1119,11 +1123,12 @@ gtk_tree_model_get_path (GtkTreeModel *tree_model,
  * @tree_model: A #GtkTreeModel.
  * @iter: The #GtkTreeIter.
  * @column: The column to lookup the value at.
- * @value: An empty #GValue to set.
+ * @value: (inout) (transfer none) An empty #GValue to set.
  *
- * Sets initializes and sets @value to that at @column.  When done with @value,
- * g_value_unset() needs to be called to free any allocated memory.
- **/
+ * Initializes and sets @value to that at @column.
+ * When done with @value, g_value_unset() needs to be called 
+ * to free any allocated memory.
+ */
 void
 gtk_tree_model_get_value (GtkTreeModel *tree_model,
 			  GtkTreeIter  *iter,
@@ -1171,10 +1176,10 @@ gtk_tree_model_iter_next (GtkTreeModel  *tree_model,
  * gtk_tree_model_iter_children:
  * @tree_model: A #GtkTreeModel.
  * @iter: The new #GtkTreeIter to be set to the child.
- * @parent: The #GtkTreeIter, or %NULL
+ * @parent: (allow-none): The #GtkTreeIter, or %NULL
  *
- * Sets @iter to point to the first child of @parent.  If @parent has no 
- * children, %FALSE is returned and @iter is set to be invalid.  @parent 
+ * Sets @iter to point to the first child of @parent.  If @parent has no
+ * children, %FALSE is returned and @iter is set to be invalid.  @parent
  * will remain a valid node after this function has been called.
  *
  * If @parent is %NULL returns the first node, equivalent to
@@ -1227,7 +1232,7 @@ gtk_tree_model_iter_has_child (GtkTreeModel *tree_model,
 /**
  * gtk_tree_model_iter_n_children:
  * @tree_model: A #GtkTreeModel.
- * @iter: The #GtkTreeIter, or %NULL.
+ * @iter: (allow-none): The #GtkTreeIter, or %NULL.
  *
  * Returns the number of children that @iter has.  As a special case, if @iter
  * is %NULL, then the number of toplevel nodes is returned.
@@ -1252,7 +1257,7 @@ gtk_tree_model_iter_n_children (GtkTreeModel *tree_model,
  * gtk_tree_model_iter_nth_child:
  * @tree_model: A #GtkTreeModel.
  * @iter: The #GtkTreeIter to set to the nth child.
- * @parent: The #GtkTreeIter to get the child from, or %NULL.
+ * @parent: (allow-none): The #GtkTreeIter to get the child from, or %NULL.
  * @n: Then index of the desired child.
  *
  * Sets @iter to be the child of @parent, using the given index.  The first
@@ -1462,7 +1467,7 @@ gtk_tree_model_get_valist (GtkTreeModel *tree_model,
  * @path: A #GtkTreePath pointing to the changed row
  * @iter: A valid #GtkTreeIter pointing to the changed row
  * 
- * Emits the "row_changed" signal on @tree_model.
+ * Emits the "row-changed" signal on @tree_model.
  **/
 void
 gtk_tree_model_row_changed (GtkTreeModel *tree_model,
@@ -1482,7 +1487,7 @@ gtk_tree_model_row_changed (GtkTreeModel *tree_model,
  * @path: A #GtkTreePath pointing to the inserted row
  * @iter: A valid #GtkTreeIter pointing to the inserted row
  * 
- * Emits the "row_inserted" signal on @tree_model
+ * Emits the "row-inserted" signal on @tree_model
  **/
 void
 gtk_tree_model_row_inserted (GtkTreeModel *tree_model,
@@ -1502,7 +1507,7 @@ gtk_tree_model_row_inserted (GtkTreeModel *tree_model,
  * @path: A #GtkTreePath pointing to the changed row
  * @iter: A valid #GtkTreeIter pointing to the changed row
  * 
- * Emits the "row_has_child_toggled" signal on @tree_model.  This should be
+ * Emits the "row-has-child-toggled" signal on @tree_model.  This should be
  * called by models after the child state of a node changes.
  **/
 void
@@ -1522,7 +1527,7 @@ gtk_tree_model_row_has_child_toggled (GtkTreeModel *tree_model,
  * @tree_model: A #GtkTreeModel
  * @path: A #GtkTreePath pointing to the previous location of the deleted row.
  * 
- * Emits the "row_deleted" signal on @tree_model.  This should be called by
+ * Emits the "row-deleted" signal on @tree_model.  This should be called by
  * models after a row has been removed.  The location pointed to by @path 
  * should be the location that the row previously was at.  It may not be a 
  * valid location anymore.
@@ -1548,7 +1553,7 @@ gtk_tree_model_row_deleted (GtkTreeModel *tree_model,
  *      to its old position before the re-ordering,
  *      i.e. @new_order<literal>[newpos] = oldpos</literal>.
  * 
- * Emits the "rows_reordered" signal on @tree_model.  This should be called by
+ * Emits the "rows-reordered" signal on @tree_model.  This should be called by
  * models when their rows have been reordered.  
  **/
 void
